@@ -53,6 +53,8 @@ gboolean          bolt_domain_is_stored (BoltDomain *domain);
 
 gboolean          bolt_domain_is_connected (BoltDomain *domain);
 
+gboolean          bolt_domain_has_iommu (BoltDomain *domain);
+
 void              bolt_domain_export (BoltDomain      *domain,
                                       GDBusConnection *connection);
 
@@ -64,6 +66,9 @@ void              bolt_domain_disconnected (BoltDomain *domain);
 void              bolt_domain_update_from_udev (BoltDomain         *domain,
                                                 struct udev_device *udev);
 
+gboolean          bolt_domain_can_delete (BoltDomain *domain,
+                                          GError    **error);
+
 /* boot acl related functions */
 gboolean          bolt_domain_supports_bootacl (BoltDomain *domain);
 
