@@ -23,11 +23,13 @@
 #include "bolt-power.h"
 
 #include "bolt-config.h"
+#include "bolt-dbus.h"
 #include "bolt-enums.h"
 #include "bolt-error.h"
 #include "bolt-fs.h"
 #include "bolt-log.h"
 #include "bolt-io.h"
+#include "bolt-reaper.h"
 #include "bolt-str.h"
 #include "bolt-unix.h"
 
@@ -42,513 +44,11 @@
 #define DEFAULT_STATEDIR "power"
 #define STATE_FILENAME "on"
 
-typedef struct udev_device udev_device;
-G_DEFINE_AUTOPTR_CLEANUP_FUNC (udev_device, udev_device_unref);
+/* prototypes */
+static void       bolt_power_release (BoltPower *power,
+                                      BoltGuard *guard);
 
-static void       bolt_power_release (BoltPower      *power,
-                                      BoltPowerGuard *guard);
-
-/* BoltPowerGuard  */
-static void       bolt_power_guard_remove (BoltPowerGuard *guard);
-
-struct _BoltPowerGuard
-{
-  GObject object;
-
-  /* book-keeping */
-  BoltPower *power;
-  char      *path;
-
-  char      *fifo;
-  guint      watch;
-
-  /* properties */
-  char *id;
-  char *who;
-  pid_t pid;
-};
-
-enum {
-  PROP_GUARD_0,
-
-  PROP_POWER,
-  PROP_PATH,
-  PROP_FIFO,
-
-  PROP_ID,
-  PROP_WHO,
-  PROP_PID,
-
-
-  PROP_GUARD_LAST
-};
-
-static GParamSpec *guard_props[PROP_GUARD_LAST] = { NULL, };
-
-G_DEFINE_TYPE (BoltPowerGuard,
-               bolt_power_guard,
-               G_TYPE_OBJECT);
-
-static void
-bolt_power_guard_finalize (GObject *object)
-{
-  BoltPowerGuard *guard = BOLT_POWER_GUARD (object);
-
-  /* remove our state file */
-  bolt_power_guard_remove (guard);
-
-  /* release the lock we have to force power,
-   * we must be intact for method call */
-  bolt_power_release (guard->power, guard);
-
-  if (guard->watch)
-    g_source_remove (guard->watch);
-
-  g_clear_pointer (&guard->fifo, g_free);
-  g_clear_pointer (&guard->who, g_free);
-  g_clear_pointer (&guard->id, g_free);
-  g_clear_object (&guard->power);
-
-  G_OBJECT_CLASS (bolt_power_guard_parent_class)->finalize (object);
-}
-
-static void
-bolt_power_guard_get_property (GObject    *object,
-                               guint       prop_id,
-                               GValue     *value,
-                               GParamSpec *pspec)
-{
-  BoltPowerGuard *guard = BOLT_POWER_GUARD (object);
-
-  switch (prop_id)
-    {
-    case PROP_POWER:
-      g_value_set_object (value, guard->power);
-      break;
-
-    case PROP_PATH:
-      g_value_set_string (value, guard->path);
-      break;
-
-    case PROP_FIFO:
-      g_value_set_string (value, guard->fifo);
-      break;
-
-    case PROP_ID:
-      g_value_set_string (value, guard->id);
-      break;
-
-    case PROP_WHO:
-      g_value_set_string (value, guard->who);
-      break;
-
-    case PROP_PID:
-      g_value_set_ulong (value, guard->pid);
-      break;
-
-    default:
-      G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
-    }
-}
-
-
-static void
-bolt_power_guard_set_property (GObject      *object,
-                               guint         prop_id,
-                               const GValue *value,
-                               GParamSpec   *pspec)
-{
-  BoltPowerGuard *guard = BOLT_POWER_GUARD (object);
-
-  switch (prop_id)
-    {
-    case PROP_POWER:
-      guard->power = g_value_dup_object (value);
-      break;
-
-    case PROP_PATH:
-      guard->path = g_value_dup_string (value);
-      break;
-
-    case PROP_FIFO:
-      guard->fifo = g_value_dup_string (value);
-      break;
-
-    case PROP_ID:
-      guard->id = g_value_dup_string (value);
-      break;
-
-    case PROP_WHO:
-      guard->who = g_value_dup_string (value);
-      break;
-
-    case PROP_PID:
-      guard->pid = g_value_get_ulong (value);
-      break;
-
-    default:
-      G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
-    }
-}
-
-
-static void
-bolt_power_guard_init (BoltPowerGuard *power)
-{
-}
-
-static void
-bolt_power_guard_class_init (BoltPowerGuardClass *klass)
-{
-  GObjectClass *gobject_class = G_OBJECT_CLASS (klass);
-
-  gobject_class->finalize = bolt_power_guard_finalize;
-  gobject_class->get_property = bolt_power_guard_get_property;
-  gobject_class->set_property = bolt_power_guard_set_property;
-
-  guard_props[PROP_POWER] =
-    g_param_spec_object ("power",
-                         NULL, NULL,
-                         BOLT_TYPE_POWER,
-                         G_PARAM_READWRITE |
-                         G_PARAM_CONSTRUCT_ONLY |
-                         G_PARAM_STATIC_STRINGS);
-
-  guard_props[PROP_PATH] =
-    g_param_spec_string ("path",
-                         NULL, NULL,
-                         NULL,
-                         G_PARAM_READWRITE |
-                         G_PARAM_CONSTRUCT_ONLY |
-                         G_PARAM_STATIC_STRINGS);
-
-  guard_props[PROP_FIFO] =
-    g_param_spec_string ("fifo",
-                         NULL, NULL,
-                         NULL,
-                         G_PARAM_READWRITE |
-                         G_PARAM_CONSTRUCT_ONLY |
-                         G_PARAM_STATIC_STRINGS);
-
-
-  guard_props[PROP_ID] =
-    g_param_spec_string ("id",
-                         NULL, NULL,
-                         NULL,
-                         G_PARAM_READWRITE |
-                         G_PARAM_CONSTRUCT_ONLY |
-                         G_PARAM_STATIC_STRINGS);
-
-  guard_props[PROP_WHO] =
-    g_param_spec_string ("who",
-                         NULL, NULL,
-                         NULL,
-                         G_PARAM_READWRITE |
-                         G_PARAM_CONSTRUCT_ONLY |
-                         G_PARAM_STATIC_STRINGS);
-
-  guard_props[PROP_PID] =
-    g_param_spec_ulong ("pid",
-                        NULL, NULL,
-                        0, G_MAXULONG, 0,
-                        G_PARAM_READWRITE |
-                        G_PARAM_CONSTRUCT_ONLY |
-                        G_PARAM_STATIC_STRINGS);
-
-  g_object_class_install_properties (gobject_class,
-                                     PROP_GUARD_LAST,
-                                     guard_props);
-}
-
-static gboolean
-bolt_power_guard_save (BoltPowerGuard *guard,
-                       GFile          *guarddir,
-                       GError        **error)
-{
-  g_autoptr(GFile) guardfile = NULL;
-  g_autoptr(GKeyFile) kf = NULL;
-  g_autofree char *path = NULL;
-  g_autofree char *name = NULL;
-  gboolean ok;
-
-  name = g_strdup_printf ("%s.guard", guard->id);
-  guardfile = g_file_get_child (guarddir, name);
-  path = g_file_get_path (guardfile);
-
-  kf = g_key_file_new ();
-
-  g_key_file_set_string (kf, "guard", "id", guard->id);
-  g_key_file_set_string (kf, "guard", "who", guard->who);
-  g_key_file_set_uint64 (kf, "guard", "pid", guard->pid);
-
-  ok = g_key_file_save_to_file (kf, path, error);
-
-  if (ok)
-    {
-      guard->path = g_steal_pointer (&path);
-      g_object_notify_by_pspec (G_OBJECT (guard),
-                                guard_props[PROP_PATH]);
-    }
-
-  return ok;
-}
-
-static BoltPowerGuard *
-bolt_power_guard_load (BoltPower  *power,
-                       const char *name,
-                       GError    **error)
-{
-  g_autoptr(GError) err = NULL;
-  g_autoptr(GFile) guardfile = NULL;
-  g_autoptr(GKeyFile) kf = NULL;
-  g_autofree char *path = NULL;
-  g_autofree char *who = NULL;
-  g_autofree char *id = NULL;
-  g_autofree char *fifo = NULL;
-  GFile *statedir;
-  gboolean ok;
-  gulong pid;
-
-  statedir = bolt_power_get_statedir (power);
-  guardfile = g_file_get_child (statedir, name);
-  path = g_file_get_path (guardfile);
-
-  kf = g_key_file_new ();
-  ok = g_key_file_load_from_file (kf, path, 0, error);
-
-  if (!ok)
-    return NULL;
-
-  id = g_key_file_get_string (kf, "guard", "id", &err);
-
-  if (id == NULL)
-    {
-      g_set_error (error, BOLT_ERROR, BOLT_ERROR_FAILED,
-                   "could not read 'id' field: %s", err->message);
-      return NULL;
-    }
-
-  who = g_key_file_get_string (kf, "guard", "who", &err);
-  if (who == NULL)
-    {
-      g_set_error_literal (error, BOLT_ERROR, BOLT_ERROR_FAILED,
-                           "field missing ('who')");
-      return NULL;
-    }
-
-  pid = (gulong) g_key_file_get_uint64 (kf, "guard", "pid", &err);
-  if (err != NULL)
-    {
-      g_set_error_literal (error, BOLT_ERROR, BOLT_ERROR_FAILED,
-                           "field missing ('pid')");
-      return NULL;
-    }
-
-  fifo = g_strdup_printf ("%s.fifo", path);
-
-  if (!g_file_test (fifo, G_FILE_TEST_EXISTS))
-    g_clear_pointer (&fifo, g_free);
-
-  return g_object_new (BOLT_TYPE_POWER_GUARD,
-                       "power", power,
-                       "path", path,
-                       "fifo", fifo,
-                       "id", id,
-                       "who", who,
-                       "pid", pid,
-                       NULL);
-}
-
-static void
-bolt_power_guard_remove (BoltPowerGuard *guard)
-{
-  g_autoptr(GError) err = NULL;
-  g_autofree char *parent = NULL;
-  gboolean ok;
-
-  /* we are not saved */
-  if (guard->path == NULL)
-    return;
-
-  /*  */
-  if (guard->fifo != NULL)
-    {
-      bolt_debug (LOG_TOPIC ("power"),
-                  "not removing guard '%s' with active fifo",
-                  guard->id);
-      return;
-    }
-
-  ok = bolt_unlink (guard->path, &err);
-  if (!ok)
-    {
-      bolt_warn_err (err, LOG_TOPIC ("power"),
-                     "Could not remove power guard: '%s' @ %s",
-                     guard->id, guard->path);
-      return;
-    }
-
-  /* we try to remove the parent dir, which will most
-   * likely fail, because it is not empty, but that we
-   * just ignore
-   */
-  parent = g_path_get_dirname (guard->path);
-  (void) rmdir (parent);
-
-  g_clear_pointer (&guard->path, g_free);
-  g_object_notify_by_pspec (G_OBJECT (guard),
-                            guard_props[PROP_PATH]);
-}
-
-static void
-bolt_power_guard_fifo_cleanup (BoltPowerGuard *guard)
-{
-  g_autoptr(GError) err = NULL;
-  gboolean ok;
-
-  if (guard->fifo == NULL)
-    return;
-
-  ok = bolt_unlink (guard->fifo, &err);
-  if (!ok)
-    {
-      bolt_warn_err (err, LOG_TOPIC ("power"),
-                     "Could not remove FIFO for power guard: '%s' @ %s",
-                     guard->id, guard->fifo);
-    }
-
-  g_clear_pointer (&guard->fifo, g_free);
-  g_object_notify_by_pspec (G_OBJECT (guard), guard_props[PROP_FIFO]);
-}
-
-static gboolean
-bolt_power_guard_mkfifo (BoltPowerGuard *guard,
-                         GError        **error)
-{
-  g_autoptr(GError) err = NULL;
-  int r;
-
-  g_return_val_if_fail (BOLT_IS_POWER_GUARD (guard), FALSE);
-  g_return_val_if_fail (error == NULL || *error == NULL, FALSE);
-
-  if (guard->fifo == NULL)
-    guard->fifo = g_strdup_printf ("%s.fifo", guard->path);
-
-  r = bolt_mkfifo (guard->fifo, 0600, &err);
-  if (r == -1 && !bolt_err_exists (err))
-    return bolt_error_propagate (error, &err);
-
-  g_object_notify_by_pspec (G_OBJECT (guard),
-                            guard_props[PROP_FIFO]);
-
-  return TRUE;
-}
-
-static gboolean
-power_guard_has_event (GIOChannel  *source,
-                       GIOCondition condition,
-                       gpointer     data)
-{
-  BoltPowerGuard *guard = data;
-
-  bolt_info (LOG_TOPIC ("power"), "got event for guard '%s' (%x)",
-             guard->id, (guint) condition);
-  guard->watch = 0;
-  return FALSE;
-}
-
-static void
-guard_watch_release (gpointer data)
-{
-  BoltPowerGuard *guard = data;
-
-  if (guard->fifo == NULL)
-    {
-      bolt_bug (LOG_TOPIC ("power"), "FIFO event but no FIFO");
-      return;
-    }
-
-  bolt_power_guard_fifo_cleanup (guard);
-
-  bolt_debug (LOG_TOPIC ("power"),
-              "released watch reference for guard '%s'",
-              guard->id);
-
-  g_object_unref (guard);
-}
-
-int
-bolt_power_guard_monitor (BoltPowerGuard *guard,
-                          GError        **error)
-{
-  g_autoptr(GIOChannel) ch = NULL;
-  gboolean ok;
-  int fd;
-
-  g_return_val_if_fail (BOLT_IS_POWER_GUARD (guard), -1);
-  g_return_val_if_fail (error == NULL || *error == NULL, -1);
-
-  ok = bolt_power_guard_mkfifo (guard, error);
-  if (!ok)
-    return -1;
-
-  /* reader */
-  fd = bolt_open (guard->fifo, O_RDONLY | O_CLOEXEC | O_NONBLOCK, 0, error);
-  if (fd == -1)
-    return -1;
-
-  ch = g_io_channel_unix_new (fd);
-
-  g_io_channel_set_close_on_unref (ch, TRUE);
-  g_io_channel_set_encoding (ch, NULL, NULL);
-  g_io_channel_set_buffered (ch, FALSE);
-  g_io_channel_set_flags (ch, G_IO_FLAG_NONBLOCK, NULL);
-  fd = -1; /* the GIOChannel owns the fd, via _close_on_unref () */
-
-  /* writer */
-  fd = bolt_open (guard->fifo, O_WRONLY | O_CLOEXEC | O_NONBLOCK, 0, error);
-  if (fd == -1)
-    return -1;
-
-  /* NB: we take a ref to the guard here */
-  guard->watch = g_io_add_watch_full (ch,
-                                      G_PRIORITY_DEFAULT,
-                                      G_IO_HUP | G_IO_ERR,
-                                      power_guard_has_event,
-                                      g_object_ref (guard),
-                                      guard_watch_release);
-
-  return fd;
-}
-
-const char *
-bolt_power_guard_get_id (BoltPowerGuard *guard)
-{
-  g_return_val_if_fail (BOLT_IS_POWER_GUARD (guard), NULL);
-
-  return guard->id;
-}
-
-const char *
-bolt_power_guard_get_who (BoltPowerGuard *guard)
-{
-  g_return_val_if_fail (BOLT_IS_POWER_GUARD (guard), NULL);
-
-  return guard->who;
-}
-
-guint
-bolt_power_guard_get_pid (BoltPowerGuard *guard)
-{
-  g_return_val_if_fail (BOLT_IS_POWER_GUARD (guard), 0);
-
-  return (guint) guard->pid;
-}
-
-/* ****************************************************************** */
-/* BoltPower */
-
-/* gobject */
+/* GObject */
 static void      power_initable_iface_init (GInitableIface *iface);
 
 static gboolean  bolt_power_initialize (GInitable    *initable,
@@ -570,9 +70,6 @@ static gboolean  bolt_power_switch_toggle (BoltPower *power,
 /* callbacks and signals */
 static gboolean bolt_power_wait_timeout (gpointer user_data);
 
-static gboolean bolt_power_reaper_timeout (gpointer user_data);
-
-
 static void     handle_uevent_udev (BoltUdev           *udev,
                                     const char         *action,
                                     struct udev_device *device,
@@ -605,7 +102,6 @@ struct _BoltPower
    * or NULL if force power is unavailable */
   char          *path;
   BoltPowerState state;
-  guint          reaper;
 
   /*  */
   guint16     guard_num;
@@ -648,9 +144,6 @@ bolt_power_finalize (GObject *object)
       bolt_power_wait_timeout (power);
     }
 
-  if (power->reaper != 0)
-    g_source_remove (power->reaper);
-
   g_clear_pointer (&power->runpath, g_free);
   g_clear_object (&power->statedir);
   g_clear_object (&power->statefile);
@@ -839,7 +332,7 @@ bolt_power_initialize (GInitable    *initable,
                        GError      **error)
 {
   g_autoptr(GError) err = NULL;
-  g_autoptr(BoltPowerGuard) guard = NULL;
+  g_autoptr(BoltGuard) guard = NULL;
   g_autofree char *statedir = NULL;
   BoltPower *power = BOLT_POWER (initable);
   gboolean on = FALSE;
@@ -913,74 +406,33 @@ static gboolean
 bolt_power_recover_guards (BoltPower *power,
                            GError   **error)
 {
-  g_autoptr(GError) err = NULL;
-  g_autoptr(GDir) dir   = NULL;
+  g_autoptr(GPtrArray) guards = NULL;
   g_autofree char *statedir = NULL;
-  const char *name;
 
   statedir = g_file_get_path (power->statedir);
 
-  dir = g_dir_open (statedir, 0, &err);
-  if (dir == NULL)
+  guards = bolt_guard_recover (statedir, error);
+  if (guards == NULL)
     return FALSE;
 
-  while ((name = g_dir_read_name (dir)) != NULL)
+  for (guint i = 0; i < guards->len; i++)
     {
-      BoltPowerGuard *guard;
-
-      if (!g_str_has_suffix (name, ".guard"))
-        continue;
-
-      guard = bolt_power_guard_load (power, name, &err);
-
-      if (guard == NULL)
-        {
-          bolt_warn_err (err, LOG_TOPIC ("power"),
-                         "could not load guard '%s'", name);
-          g_clear_error (&err);
-          continue;
-        }
-
-      /* internal guards are discarded */
-      if (bolt_streq (guard->who, "boltd"))
-        {
-          bolt_info (LOG_TOPIC ("power"), "ignoring boltd guard");
-          continue;
-        }
-      else if (!bolt_pid_is_alive (guard->pid))
-        {
-          bolt_info (LOG_TOPIC ("power"),
-                     "ignoring guard '%s for '%s': process dead",
-                     guard->id, guard->who);
-          bolt_power_guard_fifo_cleanup (guard);
-          continue;
-        }
-
-      if (guard->fifo)
-        {
-          int fd;
-          fd = bolt_power_guard_monitor (guard, &err);
-
-          if (fd < 0)
-            {
-              bolt_warn_err (err, "could not monitor guard '%d'",
-                             guard->id);
-              g_clear_error (&err);
-            }
-          else
-            {
-              (void) close (fd);
-
-              /* monitoring adds a reference that we don't want */
-              g_object_unref (guard);
-            }
-        }
+      BoltGuard *guard = g_ptr_array_index (guards, i);
+      const char *id = bolt_guard_get_id (guard);
+      const char *who = bolt_guard_get_who (guard);
+      const guint pid = bolt_guard_get_pid (guard);
 
       bolt_info (LOG_TOPIC ("power"),
-                 "guard '%s' for '%s' (pid %lu) recovered",
-                 guard->id, guard->who, (gulong) guard->pid);
+                 "guard '%s' for '%s' (pid %u) recovered",
+                 id, who, pid);
 
-      g_hash_table_insert (power->guards, guard->id, guard);
+      g_signal_connect_object (guard, "released",
+                               (GCallback) bolt_power_release,
+                               power, G_CONNECT_SWAPPED);
+
+      g_hash_table_insert (power->guards,
+                           (gpointer) bolt_guard_get_id (guard),
+                           (gpointer) g_object_ref (guard));
     }
 
   return TRUE;
@@ -1009,42 +461,6 @@ bolt_power_wait_timeout (gpointer user_data)
   return G_SOURCE_REMOVE;
 }
 
-static gboolean
-bolt_power_reaper_timeout (gpointer user_data)
-{
-  g_autoptr(GList) keys = NULL;
-  BoltPower *power = user_data;
-
-  bolt_debug (LOG_TOPIC ("power"), "looking for dead processes");
-
-  if (g_hash_table_size (power->guards) == 0)
-    {
-      bolt_debug (LOG_TOPIC ("power"), "reaper done");
-      power->reaper = 0;
-      return FALSE;
-    }
-
-  keys = g_hash_table_get_keys (power->guards);
-
-  for (GList *l = keys; l != NULL; l = l->next)
-    {
-      gpointer id = l->data;
-      BoltPowerGuard *g = g_hash_table_lookup (power->guards, id);
-
-      if (bolt_pid_is_alive (g->pid))
-        continue;
-
-      bolt_info (LOG_TOPIC ("power"),
-                 "process '%lu' is dead, "
-                 "releasing the guard '%s' for '%s'",
-                 (gulong) g->pid, g->id, g->who);
-
-      g_object_unref (g);
-    }
-
-  return TRUE;
-}
-
 static void
 handle_uevent_thunderbolt (BoltPower          *power,
                            const char         *action,
@@ -1249,21 +665,25 @@ bolt_power_gen_guard_id (BoltPower *power,
 }
 
 static void
-bolt_power_release (BoltPower *power, BoltPowerGuard *guard)
+bolt_power_release (BoltPower *power, BoltGuard *guard)
 {
+  const char *id;
+  const char *who;
   gboolean ok;
 
-  ok = g_hash_table_remove (power->guards, guard->id);
+  id = bolt_guard_get_id (guard);
+  who = bolt_guard_get_who (guard);
+
+  ok = g_hash_table_remove (power->guards, id);
 
   if (!ok)
     {
-      bolt_bug ("inactive guard ('%s', '%s') found",
-                guard->id, guard->who);
+      bolt_bug ("inactive guard ('%s', '%s') found", id, who);
       return;
     }
 
   bolt_info (LOG_TOPIC ("power"), "guard '%s' for '%s' deactivated",
-             guard->id, guard->who);
+             id, who);
 
   /* we still have active guards */
   if (g_hash_table_size (power->guards) != 0)
@@ -1296,54 +716,42 @@ handle_force_power (BoltExported          *object,
                     GDBusMethodInvocation *invocation,
                     GError               **error)
 {
-  g_autoptr(BoltPowerGuard) guard = NULL;
+  g_autoptr(BoltGuard) guard = NULL;
   g_autoptr(GUnixFDList) fds = NULL;
-  g_autoptr(GError) err = NULL;
-  g_autoptr(GVariant) res = NULL;
-  GDBusConnection *con;
   BoltPower *power;
-  const char *sender;
   const char *flags;
   const char *who;
+  gboolean ok;
   guint pid;
   int fd;
 
   power = BOLT_POWER (object);
 
-  con = g_dbus_method_invocation_get_connection (invocation);
-  sender = g_dbus_method_invocation_get_sender (invocation);
-
-  res = g_dbus_connection_call_sync (con,
-                                     "org.freedesktop.DBus",
-                                     "/",
-                                     "org.freedesktop.DBus",
-                                     "GetConnectionUnixProcessID",
-                                     g_variant_new ("(s)", sender),
-                                     G_VARIANT_TYPE ("(u)"),
-                                     G_DBUS_CALL_FLAGS_NONE,
-                                     -1, NULL,
-                                     &err);
-
-  if (res == NULL)
-    {
-      g_set_error (error, BOLT_ERROR, BOLT_ERROR_FAILED,
-                   "could not get pid of caller: %s",
-                   err->message);
-      return NULL;
-    }
-
-  g_variant_get (res, "(u)", &pid);
+  ok = bolt_dbus_get_sender_pid (invocation, &pid, error);
+  if (!ok)
+    return NULL;
 
   g_variant_get (params, "(&s&s)", &who, &flags);
 
-  /* TODO: log errors */
   guard = bolt_power_acquire_full (power, who, (pid_t) pid, error);
   if (guard == NULL)
-    return NULL;
+    {
+      bolt_warn_err (*error, LOG_TOPIC ("power"),
+                     "failed to acquire power for %s (pid %u)",
+                     who, pid);
+      return NULL;
+    }
 
-  fd = bolt_power_guard_monitor (guard, error);
+  /* monitor will add a reference to guard, so freeing one
+   * via the auto pointer is expected and in fact desired */
+  fd = bolt_guard_monitor (guard, error);
   if (fd == -1)
-    return NULL;
+    {
+      bolt_warn_err (*error, LOG_TOPIC ("power"),
+                     "failed to monitor guard %s for %s (pid %u)",
+                     bolt_guard_get_id (guard), who, pid);
+      return NULL;
+    }
 
   fds = g_unix_fd_list_new_from_array (&fd, 1);
   g_dbus_method_invocation_return_value_with_unix_fd_list (invocation,
@@ -1369,11 +777,11 @@ handle_list_guards (BoltExported          *object,
 
   for (GList *l = guards; l != NULL; l = l->next)
     {
-      BoltPowerGuard *g = BOLT_POWER_GUARD (l->data);
+      BoltGuard *g = BOLT_GUARD (l->data);
       g_variant_builder_add (&b, "(ssu)",
-                             bolt_power_guard_get_id (g),
-                             bolt_power_guard_get_who (g),
-                             bolt_power_guard_get_pid (g));
+                             bolt_guard_get_id (g),
+                             bolt_guard_get_who (g),
+                             bolt_guard_get_pid (g));
     }
 
   return g_variant_new ("(a(ssu))", &b);
@@ -1417,7 +825,7 @@ bolt_power_get_state (BoltPower *power)
   return power->state;
 }
 
-BoltPowerGuard *
+BoltGuard *
 bolt_power_acquire (BoltPower *power,
                     GError   **error)
 {
@@ -1426,7 +834,7 @@ bolt_power_acquire (BoltPower *power,
   return bolt_power_acquire_full (power, who, 0, error);
 }
 
-BoltPowerGuard *
+BoltGuard *
 bolt_power_acquire_full (BoltPower  *power,
                          const char *who,
                          pid_t       pid,
@@ -1434,7 +842,7 @@ bolt_power_acquire_full (BoltPower  *power,
 {
   g_autoptr(GError) err = NULL;
   g_autofree char *id = NULL;
-  BoltPowerGuard *guard;
+  BoltGuard *guard;
   gboolean ok;
 
   g_return_val_if_fail (BOLT_IS_POWER (power), NULL);
@@ -1475,33 +883,33 @@ bolt_power_acquire_full (BoltPower  *power,
   if (pid == 0)
     pid = getpid ();
 
-  guard = g_object_new (BOLT_TYPE_POWER_GUARD,
-                        "power", power,
+  guard = g_object_new (BOLT_TYPE_GUARD,
                         "id", id,
                         "who", who,
                         "pid", pid,
                         NULL);
 
-  /* NB: we don't take a ref here, because we want the
-   * guard to act as RAII guard, i.e. when the client
-   * releases the last reference to the guard, we call
-   * the _release() function in the finalizer */
-  g_hash_table_insert (power->guards, guard->id, guard);
+  g_signal_connect_object (guard, "released",
+                           (GCallback) bolt_power_release,
+                           power, G_CONNECT_SWAPPED);
+
+  /* NB: we don't take a ref here, because we want the guard to
+   * act as RAII guard, i.e. when the client releases the last
+   * reference to the guard, the "released" signal will be
+   * triggered and thus bolt_power_release() will be called */
+  g_hash_table_insert (power->guards,
+                       (gpointer) bolt_guard_get_id (guard),
+                       guard);
 
   bolt_info (LOG_TOPIC ("power"), "guard '%s' for '%s' active",
-             guard->id, guard->who);
-
-  if (power->reaper == 0)
-    power->reaper = g_timeout_add_seconds (POWER_REAPER_TIMEOUT,
-                                           bolt_power_reaper_timeout,
-                                           power);
+             id, who);
 
   /* guard is saved so we can recover our state if we
    * were to crash or restarted */
-  ok = bolt_power_guard_save (guard, power->statedir, &err);
+  ok = bolt_guard_save (guard, power->statedir, &err);
   if (!ok)
     bolt_warn_err (err, LOG_TOPIC ("power"),
-                   "could not save guard '%s'", guard->id);
+                   "could not save guard '%s'", id);
 
   return guard;
 }
