@@ -583,7 +583,7 @@ bolt_sysfs_write_boot_acl (const char *device,
   val = g_strjoinv (",", acl);
   path = g_build_filename (device, "boot_acl", NULL);
 
-  return bolt_file_write_all (path, val, -1, error);
+  return bolt_file_write_all (path, val, strlen (val), error);
 }
 
 gboolean
@@ -622,6 +622,7 @@ static struct
   {0x15de, TRUE},  // ALPINE_RIDGE_C_USBONLY_NHI
   {0x15e8, TRUE},  // TITAN_RIDGE_2C_NHI
   {0x15eb, TRUE},  // TITAN_RIDGE_4C_NHI
+  {0x162f, TRUE},  // REMBRANDT_NHI
   {0x8a0d, FALSE}, // ICL_NHI1
   {0x8a17, FALSE}, // ICL_NHI0
   {0x9a1b, FALSE}, // TGL_NHI0
