@@ -360,6 +360,7 @@ bolt_security_allows_pcie (BoltSecurity security)
 
     case BOLT_SECURITY_DPONLY:
     case BOLT_SECURITY_USBONLY:
+    case BOLT_SECURITY_NOPCIE:
     case BOLT_SECURITY_UNKNOWN:
       pcie = FALSE;
       break;
@@ -368,6 +369,51 @@ bolt_security_allows_pcie (BoltSecurity security)
   return pcie;
 }
 
+gboolean
+bolt_security_is_interactive (BoltSecurity security)
+{
+  gboolean interactive = FALSE;
+
+  switch (security)
+    {
+    case BOLT_SECURITY_USER:
+    case BOLT_SECURITY_SECURE:
+      interactive = TRUE;
+      break;
+
+    case BOLT_SECURITY_NONE:
+    case BOLT_SECURITY_DPONLY:
+    case BOLT_SECURITY_USBONLY:
+    case BOLT_SECURITY_NOPCIE:
+    case BOLT_SECURITY_UNKNOWN:
+      interactive = FALSE;
+      break;
+    }
+
+  return interactive;
+}
+
+const char *
+bolt_security_for_display (BoltSecurity security,
+                           gboolean     iommu)
+{
+  switch (security)
+    {
+    case BOLT_SECURITY_NONE:
+      return iommu ? "iommu" : "none";
+
+    case BOLT_SECURITY_USER:
+      return iommu ? "iommu+user" : "user";
+
+    case BOLT_SECURITY_SECURE:
+      return iommu ? "iommu+secure" : "secure";
+      break;
+
+    default:
+      return bolt_security_to_string (security);
+    }
+}
+
 BoltPolicy
 bolt_policy_from_string (const char *str)
 {
@@ -404,6 +450,12 @@ bolt_device_type_is_host (BoltDeviceType type)
   return type == BOLT_DEVICE_HOST;
 }
 
+char *
+bolt_auth_mode_to_string (BoltAuthMode mode)
+{
+  return bolt_flags_to_string (BOLT_TYPE_AUTH_MODE, mode, NULL);
+}
+
 const char *
 bolt_power_state_to_string (BoltPowerState state)
 {
